
using BPA.MQTTnet.Extensions.Settings;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using MQTTnet;
using MQTTnet.Client;
using MQTTnet.Client.Connecting;
using MQTTnet.Client.Disconnecting;
using MQTTnet.Client.Options;
using Newtonsoft.Json;
using System;
using System.Text.Json.Serialization;
using System.Threading.Tasks;

namespace BPA.MQTTnet.Extensions.Extensions
{
    public static class ServiceCollectionExtension
    {

        public static IServiceCollection AddMqttClientHostedService(this IServiceCollection services, IConfiguration configuration)
        {
            var clientSettings = JsonConvert.DeserializeObject<ClientSettings>(configuration.GetSection(nameof(ClientSettings)).Value);
            services.AddSingleton(clientSettings);
            var brokerHostSettings = JsonConvert.DeserializeObject<BrokerHostSettings>(configuration.GetSection(nameof(BrokerHostSettings)).Value);
            services.AddSingleton(brokerHostSettings);
            services.AddMqttClientServiceWithConfig();
            //services.AddMqttClientServiceWithConfig(aspOptionBuilder =>
            //{
               
            //    aspOptionBuilder
            //    .WithCredentials(clientSettings.UserName, clientSettings.Password)
            //    .WithClientId(clientSettings.Id)
            //    .WithTcpServer(brokerHostSettings.Host, brokerHostSettings.Port);
            //});
            return services;
        }

        private static IServiceCollection AddMqttClientServiceWithConfig(this IServiceCollection services)
        {
            var clientSettinigs = services.BuildServiceProvider().GetService<ClientSettings>();
            var brokerHostSettings = services.BuildServiceProvider().GetService<BrokerHostSettings>();
            //services.AddSingleton(serviceProvider =>
            //{
            //    var optionBuilder = new AspCoreMqttClientOptionBuilder(serviceProvider);
            //    configure(new MqttClientOptionsBuilder());
            //    return optionBuilder.Build();
            //});
            services.AddSingleton(serviceProvider =>
            {
                var clientOptions = new MqttClientOptionsBuilder()
                 .WithTcpServer(brokerHostSettings.Host, brokerHostSettings.Port)
                 .WithCredentials(clientSettinigs.UserName, clientSettinigs.Password)
                 .WithClientId(clientSettinigs.Id)
                 .Build();
                return clientOptions;
            });
            services.AddSingleton(serviceProvider =>
            {
                var factory = new MqttFactory();
                var client = factory.CreateMqttClient();
                client.DisconnectedHandler = new MqttClientDisconnectedHandlerDelegate(async e =>
                {
                    Console.WriteLine("### MQTT 从服务端断开连接 ###");
                    await Task.Delay(TimeSpan.FromSeconds(5));

                    try
                    {
                        await client.ConnectAsync(serviceProvider.GetService<IMqttClientOptions>());
                    }
                    catch
                    {
                        Console.WriteLine("### MQTT 重连失败 ###");
                    }
                });
                client.ConnectedHandler = new MqttClientConnectedHandlerDelegate(async e =>
                {
                    Console.WriteLine("###  MQTT 连接服务端  ###");

                    await client.SubscribeAsync(new MqttTopicFilterBuilder().WithTopic("#").Build());

                    Console.WriteLine("### MQTT 连接成功 ###");
                });
                Task.WaitAny(client.ConnectAsync(serviceProvider.GetService<IMqttClientOptions>()));
                return client;
            });
            //service
            //
            //s.AddSingleton<MqttClientService>();
            //services.AddSingleton<IHostedService>(serviceProvider =>
            //{
            //    return serviceProvider.GetService<MqttClientService>();
            //});
            //services.AddSingleton<MqttClientServiceProvider>(serviceProvider =>
            //{
            //    var mqttClientService = serviceProvider.GetService<MqttClientService>();
            //    var mqttClientServiceProvider = new MqttClientServiceProvider(mqttClientService);
            //    return mqttClientServiceProvider;
            //});
            return services;
        }
    }
}
