using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using MQTTnet.client.Test.Options;
using MQTTnet.client.Test.Settings;
using MQTTnet.Client;
using MQTTnet.Client.Connecting;
using MQTTnet.Client.Disconnecting;
using MQTTnet.Client.Options;
using System;
using System.Threading.Tasks;

namespace MQTTnet.client.Test.Extensions
{
    public static class ServiceCollectionExtension
    {

        public static IServiceCollection AddMqttClientHostedService(this IServiceCollection services, IConfiguration configuration)
        {
            ClientSettings clientSettings = new ClientSettings();
            configuration.GetSection(nameof(ClientSettings)).Bind(clientSettings);
            services.AddSingleton(clientSettings);

            BrokerHostSettings brokerHostSettings = new BrokerHostSettings();
            configuration.GetSection(nameof(BrokerHostSettings)).Bind(brokerHostSettings);
            services.AddSingleton(brokerHostSettings);
            services.AddMqttClientServiceWithConfig( );
            return services;
        }

        private static IServiceCollection AddMqttClientServiceWithConfig(this IServiceCollection services)
        {

            var clientSettinigs = services.BuildServiceProvider().GetService<ClientSettings>();
            var brokerHostSettings = services.BuildServiceProvider().GetService<BrokerHostSettings>();
            //services.AddSingleton(serviceProvider =>
            //{
            //    var optionBuilder = new AspCoreMqttClientOptionBuilder(serviceProvider);
            //    configure(new MqttClientOptionsBuilder());
            //    return optionBuilder.Build();
            //});
            services.AddSingleton(serviceProvider =>
            {
                var clientOptions = new MqttClientOptionsBuilder()
                 .WithTcpServer(brokerHostSettings.Host, brokerHostSettings.Port)
                 .WithCredentials(clientSettinigs.UserName, clientSettinigs.Password)
                 .WithClientId(clientSettinigs.Id)
                 .Build();
                return clientOptions;
            });
            services.AddSingleton(serviceProvider =>
            {
                var factory = new MqttFactory();
                var client = factory.CreateMqttClient();
                client.DisconnectedHandler = new MqttClientDisconnectedHandlerDelegate(async e =>
                {
                    Console.WriteLine("### MQTT 从服务端断开连接 ###");
                    await Task.Delay(TimeSpan.FromSeconds(5));

                    try
                    {
                        await client.ConnectAsync(serviceProvider.GetService<IMqttClientOptions>());
                    }
                    catch
                    {
                        Console.WriteLine("### MQTT 重连失败 ###");
                    }
                });
                client.ConnectedHandler = new MqttClientConnectedHandlerDelegate(async e =>
                {
                    Console.WriteLine("###  MQTT 连接服务端  ###");

                    await client.SubscribeAsync(new MqttTopicFilterBuilder().WithTopic("#").Build());

                    Console.WriteLine("### MQTT 连接成功 ###");
                });
                Task.WaitAny(client.ConnectAsync(serviceProvider.GetService<IMqttClientOptions>()));
                return client;
            });
           
            return services;
        }
    }
}
