using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using Microsoft.Extensions.Logging;
using MQTTnet.Client;
using MQTTnet.Client.Connecting;
using MQTTnet.Client.Disconnecting;
using MQTTnet.Client.Options;
using MQTTnet.Client.Receiving;
using MQTTnet.Protocol;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MQTTnet.client.Test.Pages
{
    public class IndexModel : PageModel
    {
        private readonly ILogger<IndexModel> _logger;
        private IMqttClient _mqttClient;

        private IMqttClientOptions _mqttClientOptions;
        public IndexModel(ILogger<IndexModel> logger)
        {
            //, IMqttClient mqttClient, IMqttClientOptions mqttClientOptions
            //_mqttClient =mqttClient;
            //_mqttClientOptions=mqttClientOptions;
           // Task.Run(RunAsync);
            Task.Run(PublicAsync);
        }

        public async Task PublicAsync()
        {
            var applicationMessage = new MqttApplicationMessageBuilder()
                .WithTopic("test")
                .WithPayload("Hello World")
                .WithAtLeastOnceQoS()
                .Build();

            await _mqttClient.PublishAsync(applicationMessage);
        }

        public  async Task RunAsync()
        {
            try
            {
             
                IMqttClient client = _mqttClient;
                client.ApplicationMessageReceivedHandler = new MqttApplicationMessageReceivedHandlerDelegate(e =>
                {
                    Console.WriteLine("### RECEIVED APPLICATION MESSAGE ###");
                    Console.WriteLine($"+ Topic = {e.ApplicationMessage.Topic}");
                    Console.WriteLine($"+ Payload = {Encoding.UTF8.GetString(e.ApplicationMessage.Payload)}");
                    Console.WriteLine($"+ QoS = {e.ApplicationMessage.QualityOfServiceLevel}");
                    Console.WriteLine($"+ Retain = {e.ApplicationMessage.Retain}");
                   // Console.WriteLine();
                });

               
                await client.SubscribeAsync(new MqttTopicFilter { Topic = "test", QualityOfServiceLevel = MqttQualityOfServiceLevel.AtMostOnce });
                await client.SubscribeAsync(new MqttTopicFilter { Topic = "6666", QualityOfServiceLevel = MqttQualityOfServiceLevel.AtMostOnce });
               
            }
            catch (Exception exception)
            {
                Console.WriteLine(exception);
            }
        }



        public void OnGet()
        {

        }
    }
}
